@@ -14,10 +14,10 @@ import _pytest.hookspec
 import pytest
 
 
-class WorkerInteractor:
+class WorkerInteractor(object):
     def __init__(self, config, channel):
         self.config = config
-        self.workerid = config.workerinput.get('workerid', "?")
+        self.workerid = config.workerinput.get("workerid", "?")
         self.log = py.log.Producer("worker-%s" % self.workerid)
         if not config.option.debug:
             py.log.setconsumer(self.log._keywords, None)
@@ -39,7 +39,7 @@ class WorkerInteractor:
 
     @pytest.hookimpl(hookwrapper=True)
     def pytest_sessionfinish(self, exitstatus):
-        self.config.workeroutput['exitstatus'] = exitstatus
+        self.config.workeroutput["exitstatus"] = exitstatus
         yield
         self.sendevent("workerfinished", workeroutput=self.config.workeroutput)
 
@@ -56,7 +56,7 @@ class WorkerInteractor:
                 return True
             self.log("received command", name, kwargs)
             if name == "runtests":
-                torun.extend(kwargs['indices'])
+                torun.extend(kwargs["indices"])
             elif name == "runtests_all":
                 torun.extend(range(len(session.items)))
             self.log("items to run:", torun)
@@ -79,24 +79,25 @@ class WorkerInteractor:
             nextitem = None
 
         start = time.time()
-        self.config.hook.pytest_runtest_protocol(
-            item=item,
-            nextitem=nextitem)
+        self.config.hook.pytest_runtest_protocol(item=item, nextitem=nextitem)
         duration = time.time() - start
-        self.sendevent("runtest_protocol_complete", item_index=self.item_index,
-                       duration=duration)
+        self.sendevent(
+            "runtest_protocol_complete", item_index=self.item_index, duration=duration
+        )
 
     def pytest_collection_finish(self, session):
         self.sendevent(
             "collectionfinish",
             topdir=str(session.fspath),
-            ids=[item.nodeid for item in session.items])
+            ids=[item.nodeid for item in session.items],
+        )
 
     def pytest_runtest_logstart(self, nodeid, location):
         self.sendevent("logstart", nodeid=nodeid, location=location)
 
     # the pytest_runtest_logfinish hook was introduced in pytest 3.4
-    if hasattr(_pytest.hookspec, 'pytest_runtest_logfinish'):
+    if hasattr(_pytest.hookspec, "pytest_runtest_logfinish"):
+
         def pytest_runtest_logfinish(self, nodeid, location):
             self.sendevent("logfinish", nodeid=nodeid, location=location)
 
@@ -108,12 +109,19 @@ class WorkerInteractor:
         self.sendevent("testreport", data=data)
 
     def pytest_collectreport(self, report):
-        data = serialize_report(report)
-        self.sendevent("collectreport", data=data)
+        # master only needs reports that failed, as optimization send only them instead (#330)
+        if report.failed:
+            data = serialize_report(report)
+            self.sendevent("collectreport", data=data)
 
     def pytest_logwarning(self, message, code, nodeid, fslocation):
-        self.sendevent("logwarning", message=message, code=code, nodeid=nodeid,
-                       fslocation=str(fslocation))
+        self.sendevent(
+            "logwarning",
+            message=message,
+            code=code,
+            nodeid=nodeid,
+            fslocation=str(fslocation),
+        )
 
 
 def serialize_report(rep):
@@ -122,34 +130,33 @@ def serialize_report(rep):
         reprcrash = rep.longrepr.reprcrash.__dict__.copy()
 
         new_entries = []
-        for entry in reprtraceback['reprentries']:
-            entry_data = {
-                'type': type(entry).__name__,
-                'data': entry.__dict__.copy(),
-            }
-            for key, value in entry_data['data'].items():
-                if hasattr(value, '__dict__'):
-                    entry_data['data'][key] = value.__dict__.copy()
+        for entry in reprtraceback["reprentries"]:
+            entry_data = {"type": type(entry).__name__, "data": entry.__dict__.copy()}
+            for key, value in entry_data["data"].items():
+                if hasattr(value, "__dict__"):
+                    entry_data["data"][key] = value.__dict__.copy()
             new_entries.append(entry_data)
 
-        reprtraceback['reprentries'] = new_entries
+        reprtraceback["reprentries"] = new_entries
 
         return {
-            'reprcrash': reprcrash,
-            'reprtraceback': reprtraceback,
-            'sections': rep.longrepr.sections
+            "reprcrash": reprcrash,
+            "reprtraceback": reprtraceback,
+            "sections": rep.longrepr.sections,
         }
 
     import py
+
     d = rep.__dict__.copy()
-    if hasattr(rep.longrepr, 'toterminal'):
-        if hasattr(rep.longrepr, 'reprtraceback') \
-                and hasattr(rep.longrepr, 'reprcrash'):
-            d['longrepr'] = disassembled_report(rep)
+    if hasattr(rep.longrepr, "toterminal"):
+        if hasattr(rep.longrepr, "reprtraceback") and hasattr(
+            rep.longrepr, "reprcrash"
+        ):
+            d["longrepr"] = disassembled_report(rep)
         else:
-            d['longrepr'] = str(rep.longrepr)
+            d["longrepr"] = str(rep.longrepr)
     else:
-        d['longrepr'] = rep.longrepr
+        d["longrepr"] = rep.longrepr
     for name in d:
         if isinstance(d[name], py.path.local):
             d[name] = str(d[name])
@@ -160,6 +167,7 @@ def serialize_report(rep):
 
 def getinfodict():
     import platform
+
     return dict(
         version=sys.version,
         version_info=tuple(sys.version_info),
@@ -172,7 +180,8 @@ def getinfodict():
 
 def remote_initconfig(option_dict, args):
     from _pytest.config import Config
-    option_dict['plugins'].append("no:terminal")
+
+    option_dict["plugins"].append("no:terminal")
     config = Config.fromdictargs(option_dict, args)
     config.option.looponfail = False
     config.option.usepdb = False
@@ -183,18 +192,19 @@ def remote_initconfig(option_dict, args):
     return config
 
 
-if __name__ == '__channelexec__':
+if __name__ == "__channelexec__":
     channel = channel  # noqa
     workerinput, args, option_dict = channel.receive()
     importpath = os.getcwd()
     sys.path.insert(0, importpath)  # XXX only for remote situations
-    os.environ['PYTHONPATH'] = (
-        importpath + os.pathsep +
-        os.environ.get('PYTHONPATH', ''))
-    os.environ['PYTEST_XDIST_WORKER'] = workerinput['workerid']
-    os.environ['PYTEST_XDIST_WORKER_COUNT'] = str(workerinput['workercount'])
+    os.environ["PYTHONPATH"] = (
+        importpath + os.pathsep + os.environ.get("PYTHONPATH", "")
+    )
+    os.environ["PYTEST_XDIST_WORKER"] = workerinput["workerid"]
+    os.environ["PYTEST_XDIST_WORKER_COUNT"] = str(workerinput["workercount"])
     # os.environ['PYTHONPATH'] = importpath
     import py
+
     config = remote_initconfig(option_dict, args)
     config.workerinput = workerinput
     config.workeroutput = {}
